module File = {
  type t = {
    name: string,
    id: string,
    thumbnailLink: option(string)
  };
  type source =
    | Dropbox
    | GoogleDrive;
  let decodeDropbox = json =>
    Json.Decode.{
      name: json |> field("name", string),
      id: json |> field("path_lower", string),
      thumbnailLink: None
    };
  let decodeGoogleDrive = json =>
    Json.Decode.{
      name: json |> field("name", string),
      id: json |> field("id", string),
      thumbnailLink: json |> field("thumbnailLink", string) |> (s => Some(s))
    };
  let encode = model =>
    Json.Encode.(
      object_([
        ("name", model.name |> string),
        ("id", model.id |> string),
        ("thumbnailLink", model.thumbnailLink |> nullable(string))
      ])
    );
};

type t = {
  hasMore: bool,
  cursor: option(string),
  files: array(File.t)
};

let decode = json =>
  Json.Decode.{
    hasMore: json |> field("hasMore", bool),
    cursor: json |> optional(field("cursor", string)),
    files:
      json
      |> field(
           "files",
           array(either(File.decodeDropbox, File.decodeGoogleDrive))
         )
  };

let encode = model =>
  Json.Encode.(
    object_([
      ("hasMore", model.hasMore |> bool),
      ("cursor", model.cursor |> nullable(string)),
      ("files", model.files |> arrayOf(File.encode))
    ])
  );