open Utils;

module Params = {
  type method =
    | Get
    | Put;
  type uploadType =
    | Document
    | Avatar;
  type identifier =
    | FileId(string)
    | Url(string);
  type t = {
    method,
    uploadType: option(uploadType),
    identifier,
    contentType: string
  };
  let make = (~method, ~uploadType=?, ~identifier, ~contentType, ()) => {
    method,
    uploadType,
    identifier,
    contentType
  };
  let encode = model =>
    Json.Encode.[
      Some((
        "method",
        switch model.method {
        | Get => string("GET")
        | Put => string("PUT")
        }
      )),
      model.uploadType
      |> Js.Option.map(
           wrapBs(
             fun
             | Document => ("uploadType", string("document"))
             | Avatar => ("uploadType", string("avatar"))
           )
         ),
      switch model.identifier {
      | FileId(id) => Some(("id", string(id)))
      | Url(url) => Some(("url", string(url)))
      },
      Some(("contentType", string(model.contentType)))
    ]
    |> nullableFieldEncoder;
};

let request = params =>
  Request.(
    make(
      ~path="/gcs/sign",
      ~body=params |> Params.encode |> Js.Json.stringify,
      ()
    )
  )
  |> Vow.Result.flatMap(ResponseModel.decode(SignedUrlModel.decode));